@@ -1,7 +1,8 @@
 extern crate core;
 extern crate rand;
+extern crate regex;
 
-use rand::Rng;
+use rand::{Rng, ThreadRng};
 use std::error;
 use std::fmt;
 
@@ -13,6 +14,244 @@ pub struct Ssn {
     pub gender: Gender,
 }
 
+fn century_range(sep: &Option<char>) -> Result<Vec<usize>, GenerateError> {
+    match sep {
+        Some(c) => match from_separator(c) {
+            Ok(v) => Ok(vec![v]),
+            Err(_) => Err(GenerateError),
+        },
+        None => Ok(vec![1800usize, 1900usize, 2000usize]),
+    }
+}
+
+fn decade_range(y1: &Option<u8>) -> Vec<usize> {
+    match y1 {
+        Some(v) => vec![*v as usize],
+        None => (0usize..=9usize).collect(),
+    }
+}
+
+fn y2_range(rng: &mut ThreadRng, y2: &Option<u8>) -> Vec<usize> {
+    match y2 {
+        Some(v) => vec![*v as usize],
+        None => {
+            let mut range: Vec<usize> = (0usize..=9usize).collect();
+            rng.shuffle(&mut range);
+            range
+        }
+    }
+}
+
+fn m_range(m1: &Option<u8>, m2: &Option<u8>) -> Result<Vec<usize>, GenerateError> {
+    let range = match (m1, m2) {
+        (Some(ref m1), Some(ref m2)) => {
+            let m = (m1 * 10 + m2) as usize;
+            if !(1..=12).contains(&m) {
+                return Err(GenerateError);
+            };
+            vec![m]
+        }
+        (Some(ref m1), None) => ((if *m1 == 0 { 1 } else { 0 })..(if *m1 == 0 { 10 } else { 3 }))
+            .map(|m2| (m1 * 10 + m2) as usize)
+            .collect(),
+        (None, Some(ref m2)) => ((if *m2 == 0 { 1 } else { 0 })..2)
+            .map(|m1| (m1 * 10 + m2) as usize)
+            .collect(),
+        (None, None) => (1usize..13usize).collect(),
+    };
+    Ok(range)
+}
+
+fn d_range(d1: &Option<u8>, d2: &Option<u8>) -> Result<Vec<usize>, GenerateError> {
+    match (d1, d2) {
+        (Some(ref d1), Some(ref d2)) => {
+            let d = (d1 * 10 + d2) as usize;
+            if d < 1 {
+                return Err(GenerateError);
+            };
+            Ok(vec![d])
+        }
+        (Some(ref d1), None) => {
+            // if *d1 < 1 || *d1 as usize > days_in_month / 10 {
+            //     return Err(GenerateError);
+            // };
+            Ok(((if *d1 as usize == 0 { 1 } else { 0 })..10)
+                .map(|d2| *d1 as usize * 10 + d2)
+                .collect())
+        }
+        (None, Some(ref d2)) => {
+            Ok(((if *d2 as usize == 0 { 1 } else { 0 })
+                ..(/*if days_in_month % 10 == 3 { 4 } else { 3 }*/4))
+                .map(|d1| d1 * 10 + *d2 as usize)
+                .collect())
+        }
+        (None, None) => Ok(
+            // (1..(days_in_month + 1)).collect()
+            (1..32).collect(),
+        ),
+    }
+}
+
+pub fn generate_by_pattern_with_any_checksum(
+    pattern: &SsnPattern,
+) -> Result<String, GenerateError> {
+    let mut rng = rand::thread_rng();
+
+    let century = match pattern
+        .sep
+        .map(|c| from_separator(&c))
+        .unwrap_or_else(|| Ok(1800 + rng.gen_range(0, 3) * 100))
+    {
+        Ok(v) => v,
+        Err(_) => return Err(GenerateError),
+    };
+    let decade = pattern.y1.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
+    let y2 = pattern.y2.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
+    let year = century + decade * 10 + y2;
+
+    let month: usize = match (pattern.m1, pattern.m2) {
+        (Some(ref m1), Some(ref m2)) => {
+            let m = (m1 * 10 + m2) as usize;
+            if !(1..=12).contains(&m) {
+                return Err(GenerateError);
+            };
+            m
+        }
+        (Some(ref m1), None) => {
+            let m2 = rng.gen_range(if *m1 == 0 { 1 } else { 0 }, if *m1 == 0 { 10 } else { 3 });
+            (m1 * 10 + m2) as usize
+        }
+        (None, Some(ref m2)) => {
+            let m1 = rng.gen_range(if *m2 == 0 { 1 } else { 0 }, 2);
+            (m1 * 10 + m2) as usize
+        }
+        (None, None) => rng.gen_range(1, 13),
+    };
+
+    let days_in_month = days_in_month(month, year);
+    let day: usize = match (pattern.d1, pattern.d2) {
+        (Some(ref d1), Some(ref d2)) => {
+            let d = (d1 * 10 + d2) as usize;
+            if d < 1 || d > days_in_month {
+                return Err(GenerateError);
+            };
+            d
+        }
+        (Some(ref d1), None) => {
+            if *d1 < 1 || *d1 as usize > days_in_month / 10 {
+                return Err(GenerateError);
+            };
+            let d2 = rng.gen_range(if *d1 as usize == 0 { 1 } else { 0 }, 10);
+            *d1 as usize * 10 + d2
+        }
+        (None, Some(ref d2)) => {
+            let d1 = rng.gen_range(
+                if *d2 as usize == 0 { 1 } else { 0 },
+                if days_in_month % 10 == 3 { 4 } else { 3 },
+            ) as usize;
+            d1 * 10 + *d2 as usize
+        }
+        (None, None) => rng.gen_range(1, days_in_month + 1),
+    };
+
+    let i1 = pattern.i1.unwrap_or_else(|| rng.gen_range(0, 9)) as usize;
+    let i2 = pattern.i2.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
+    let i3 = pattern
+        .i3
+        .unwrap_or_else(|| rng.gen_range(if i1 == 0 && i2 == 0 { 2 } else { 0 }, 10))
+        as usize;
+    let identifier = i1 * 100 + i2 * 10 + i3;
+    if !(2..=899).contains(&identifier) {
+        return Err(GenerateError);
+    }
+    let nums = day * 10_000_000 + month * 100_000 + (year % 100) * 1_000 + identifier;
+    let checksum = CHECKSUM_TABLE[nums % 31];
+
+    Ok(format!(
+        "{:02.}{:02.}{:02.}{}{:03.}{}",
+        day,
+        month,
+        year % 100,
+        to_separator(century)?,
+        identifier,
+        checksum
+    ))
+}
+
+pub fn generate_by_pattern_with_fixed_checksum(
+    pattern: &SsnPattern,
+) -> Result<String, GenerateError> {
+    let mut rng = rand::thread_rng();
+    let mut centuries = century_range(&pattern.sep)?;
+    rng.shuffle(&mut centuries);
+    let mut decades = decade_range(&pattern.y1);
+    rng.shuffle(&mut decades);
+    let mut y2s = y2_range(&mut rng, &pattern.y2);
+    rng.shuffle(&mut y2s);
+    let mut months = m_range(&pattern.m1, &pattern.m2)?;
+    rng.shuffle(&mut months);
+    let mut days = d_range(&pattern.d1, &pattern.d2)?;
+    rng.shuffle(&mut days);
+    let mut i1s = pattern
+        .i1
+        .map(|v| vec![v as usize])
+        .unwrap_or_else(|| (0usize..=9usize).collect());
+    rng.shuffle(&mut i1s);
+    let mut i2s = pattern
+        .i2
+        .map(|v| vec![v as usize])
+        .unwrap_or_else(|| (0usize..=9usize).collect());
+    rng.shuffle(&mut i2s);
+    let mut i3s = pattern
+        .i3
+        .map(|v| vec![v as usize])
+        // .unwrap_or_else(|| ((if i1 == 0 && i2 == 0 { 2usize } else { 0usize })..10usize)
+        .unwrap_or_else(|| (0usize..=9usize).collect());
+    rng.shuffle(&mut i3s);
+    for century in &centuries {
+        for decade in &decades {
+            for y2 in &y2s {
+                let year = century + decade * 10 + y2;
+                for month in &months {
+                    let days_in_this_month = days_in_month(*month, year);
+                    for day in days.iter().filter(|d| d <= &&days_in_this_month) {
+                        for i1 in &i1s {
+                            for i2 in &i2s {
+                                for i3 in &i3s {
+                                    let identifier = i1 * 100 + i2 * 10 + i3;
+                                    if !(2..=899).contains(&identifier) {
+                                        continue;
+                                    }
+                                    let nums = day * 10_000_000
+                                        + month * 100_000
+                                        + (year % 100) * 1_000
+                                        + identifier;
+                                    let exp_checksum = CHECKSUM_TABLE[nums % 31];
+                                    let checksum = &pattern.check.unwrap();
+                                    if exp_checksum != *checksum {
+                                        // println!("{} != {} was not valid guess", exp_checksum, checksum);
+                                        continue;
+                                    }
+                                    return Ok(format!(
+                                        "{:02.}{:02.}{:02.}{}{:03.}{}",
+                                        day,
+                                        month,
+                                        year % 100,
+                                        to_separator(*century)?,
+                                        identifier,
+                                        checksum
+                                    ));
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    }
+    Err(GenerateError)
+}
+
 impl Ssn {
     /// Parse HETU.
     pub fn parse(ssn: &str) -> Result<Ssn, ParseError> {
@@ -22,12 +261,7 @@ impl Ssn {
         let chars: Vec<char> = ssn.chars().collect();
 
         let separator = chars[6];
-        match separator {
-            '+' => (),
-            '-' => (),
-            'A' => (),
-            _ => return Err(ParseError::Syntax("Invalid separator", 6, 7)),
-        };
+        from_separator(&separator)?;
 
         let date: usize = match ssn[0..6].parse::<usize>() {
             Ok(n) => n,
@@ -39,13 +273,7 @@ impl Ssn {
             return Err(ParseError::Month("Invalid month number", 2, 4));
         }
 
-        let year = date % 100
-            + match separator {
-                '+' => 1800,
-                '-' => 1900,
-                'A' => 2000,
-                _ => return Err(ParseError::Syntax("Invalid separator", 6, 7)),
-            };
+        let year = date % 100 + from_separator(&separator)?;
 
         let days_in_month = days_in_month(month, year);
         let day = date / 10_000;
@@ -87,12 +315,7 @@ impl Ssn {
         let year = rng.gen_range(1890, 2016);
         let month = rng.gen_range(1, 13);
         let day = rng.gen_range(1, days_in_month(month, year) + 1);
-        let separator = match year / 100 {
-            18 => "+",
-            19 => "-",
-            20 => "A",
-            _ => panic!(),
-        };
+        let separator = to_separator(year).unwrap();
         let identifier = rng.gen_range(2, 900);
         let nums = day * 10_000_000 + month * 100_000 + (year % 100) * 1_000 + identifier;
         let checksum = CHECKSUM_TABLE[nums % 31];
@@ -109,88 +332,30 @@ impl Ssn {
 
     /// Generate HETU with matching fields.
     pub fn generate_by_pattern(pattern: &SsnPattern) -> Result<String, GenerateError> {
-        let mut rng = rand::thread_rng();
+        match &pattern.check {
+            Some(_) => generate_by_pattern_with_fixed_checksum(pattern),
+            None => generate_by_pattern_with_any_checksum(pattern),
+        }
+    }
+}
 
-        let separator = pattern
-            .sep
-            .map(|c| match c {
-                '+' => 0,
-                '-' => 1,
-                'A' => 2,
-                _ => panic!(),
-            })
-            .unwrap_or_else(|| rng.gen_range(0, 3)) as usize;
-        let y1 = pattern.y1.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-        let y2 = pattern.y2.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-        let year = (1800 + separator * 100) + y1 * 10 + y2;
+/** Parse separator into century. */
+fn from_separator<'a>(separator: &char) -> Result<usize, ParseError<'a>> {
+    match separator {
+        '+' => Ok(1800),
+        '-' => Ok(1900),
+        'A' => Ok(2000),
+        _ => return Err(ParseError::Syntax("Invalid separator", 6, 7)),
+    }
+}
 
-        let m1 = pattern
-            .m1
-            .unwrap_or_else(|| rng.gen_range(if pattern.m2.unwrap_or(1) == 0 { 1 } else { 0 }, 2))
-            as usize;
-        let m2 = pattern.m2.unwrap_or_else(|| {
-            rng.gen_range(if m1 == 0 { 1 } else { 0 }, if m1 == 1 { 3 } else { 10 })
-        }) as usize;
-        let month = m1 * 10 + m2;
-
-        let days_in_month = days_in_month(month, year);
-        let d1 = pattern.d1.unwrap_or_else(|| {
-            rng.gen_range(
-                if pattern.d2.unwrap_or(1) == 0 { 1 } else { 0 },
-                if days_in_month % 10 == 3 { 4 } else { 3 },
-            )
-        }) as usize;
-        let d2 = pattern.d2.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-        let day = d1 * 10 + d2;
-
-        let (identifier, checksum) = if pattern.check.is_some() {
-            let get_numbers = |num: Option<u8>| -> Vec<u8> {
-                num.map(|v| vec![v])
-                    .unwrap_or_else(|| vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
-            };
-            let res = || -> Option<(usize, char)> {
-                let exp = pattern.check.unwrap();
-                for i1 in get_numbers(pattern.i1) {
-                    for i2 in get_numbers(pattern.i2) {
-                        for i3 in get_numbers(pattern.i3) {
-                            let identifier = i1 as usize * 100 + i2 as usize * 10 + i3 as usize;
-                            let checksum = checksum_num(day, month, year, identifier);
-                            if checksum == exp {
-                                return Some((identifier, checksum));
-                            }
-                        }
-                    }
-                }
-                None
-            }();
-            match res {
-                Some((i, c)) => (i, c),
-                None => return Err(GenerateError),
-            }
-        } else {
-            let i1 = pattern.i1.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-            let i2 = pattern.i2.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-            let i3 = pattern.i3.unwrap_or_else(|| rng.gen_range(0, 10)) as usize;
-            let identifier = i1 * 100 + i2 * 10 + i3;
-            let nums = day * 10_000_000 + month * 100_000 + (year % 100) * 1_000 + identifier;
-            let checksum = CHECKSUM_TABLE[nums % 31];
-            (identifier, checksum)
-        };
-
-        Ok(format!(
-            "{:02.}{:02.}{:02.}{}{:03.}{}",
-            day,
-            month,
-            year % 100,
-            match separator {
-                0 => '+',
-                1 => '-',
-                2 => 'A',
-                _ => panic!(),
-            },
-            identifier,
-            checksum
-        ))
+/** Get separator character for year. */
+fn to_separator(year: usize) -> Result<char, GenerateError> {
+    match year / 100 {
+        18 => Ok('+'),
+        19 => Ok('-'),
+        20 => Ok('A'),
+        _ => Err(GenerateError),
     }
 }
 
@@ -225,10 +390,7 @@ impl SsnPattern {
             check: None,
         }
     }
-    fn parse_char(
-        chars: &str, // chars: &Vec<char>,
-        index: usize,
-    ) -> Result<Option<u8>, ParseError> {
+    fn parse_char(chars: &str, index: usize) -> Result<Option<u8>, ParseError> {
         let c = chars.chars().nth(index).unwrap();
         if c == '?' {
             Ok(None)
@@ -264,8 +426,10 @@ impl SsnPattern {
         let i3: Option<u8> = SsnPattern::parse_char(&p, 9)?;
         let check: Option<char> = match p.chars().nth(10).unwrap() {
             '?' => None,
-            // TODO validate that this is checksum character
-            sep => Some(sep),
+            sep if CHECKSUM_TABLE.contains(&sep) => Some(sep),
+            _ => {
+                return Err(ParseError::Syntax("Invalid checksum character", 10, 11));
+            }
         };
         Ok(SsnPattern {
             d1,
@@ -413,10 +577,10 @@ fn checksum(ssn: &str) -> char {
     let nums: usize = (&hello).parse().unwrap();
     CHECKSUM_TABLE[nums % 31]
 }
-fn checksum_num(day: usize, month: usize, year: usize, identifier: usize) -> char {
-    let nums = day * 10_000_000 + month * 100_000 + (year % 100) * 1_000 + identifier;
-    CHECKSUM_TABLE[nums % 31]
-}
+// fn checksum_num(day: usize, month: usize, year: usize, identifier: usize) -> char {
+//     let nums = day * 10_000_000 + month * 100_000 + (year % 100) * 1_000 + identifier;
+//     CHECKSUM_TABLE[nums % 31]
+// }
 
 fn is_leap_year(year: usize) -> bool {
     ((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0)
@@ -427,52 +591,85 @@ mod tests {
     use super::*;
 
     #[test]
-    fn test_parse() {
+    fn test_parse_empty_string() {
         assert!(
             Ssn::parse("").unwrap_err() == ParseError::Syntax("Invalid length", 0, 0),
             "fail when given empty String"
         );
+    }
+    #[test]
+    fn test_parse_month_too_large() {
         assert!(
             Ssn::parse("301398-1233").unwrap_err()
                 == ParseError::Month("Invalid month number", 2, 4),
             "fail when given birthdate with month out of bounds"
         );
+    }
+    #[test]
+    fn test_parse_day_too_large() {
         assert!(
             Ssn::parse("320198-123P").unwrap_err() == ParseError::Day("Invalid day number", 0, 2),
             "fail when given birthdate with date out of bounds in January"
         );
+    }
+    #[test]
+    fn test_parse_day_too_large_on_non_leap_year() {
         assert!(
             Ssn::parse("290299-123U").unwrap_err() == ParseError::Day("Invalid day number", 0, 2),
             "fail when given birthdate with date out of bounds in February, non leap year"
         );
+    }
+    #[test]
+    fn test_parse_day_too_large_on_leap_year() {
         assert!(
             Ssn::parse("300204-123Y").unwrap_err() == ParseError::Day("Invalid day number", 0, 2),
             "fail when given birth date with date out of bounds in February, a leap year"
         );
+    }
+    #[test]
+    fn test_parse_invalid_year_characters() {
         assert!(
             Ssn::parse("0101AA-123A").unwrap_err() == ParseError::Syntax("Date not integer", 0, 6),
             "fail when given birth date with alphabets"
         );
+    }
+    #[test]
+    fn test_parse_invalid_separator() {
         assert!(
             Ssn::parse("010195_433X").unwrap_err() == ParseError::Syntax("Invalid separator", 6, 7),
             "fail when given invalid separator chars"
         );
+    }
+    #[test]
+    fn test_parse_date_too_long() {
         assert!(
             Ssn::parse("01011995+433X").unwrap_err() == ParseError::Syntax("Invalid length", 0, 13),
             "fail when given too long date"
         );
+    }
+    #[test]
+    fn test_parse_date_too_short() {
         assert!(
             Ssn::parse("01015+433X").unwrap_err() == ParseError::Syntax("Invalid length", 0, 10),
             "fail when given too short date"
         );
+    }
+    #[test]
+    fn test_parse_identifier_too_long() {
         assert!(
             Ssn::parse("010195+4433X").unwrap_err() == ParseError::Syntax("Invalid length", 0, 12),
             "fail when given too long checksum part"
         );
+    }
+    #[test]
+    fn test_parse_identifier_too_short() {
         assert!(
             Ssn::parse("010195+33X").unwrap_err() == ParseError::Syntax("Invalid length", 0, 10),
             "fail when given too long checksum part"
         );
+    }
+    #[test]
+    fn test_parse_male() {
         assert_eq!(
             Ssn::parse("010195+433X").unwrap(),
             Ssn {
@@ -482,6 +679,9 @@ mod tests {
                 gender: Gender::Male,
             }
         );
+    }
+    #[test]
+    fn test_parse_female() {
         assert_eq!(
             Ssn::parse("010197-100P").unwrap(),
             Ssn {
@@ -491,6 +691,9 @@ mod tests {
                 gender: Gender::Female,
             }
         );
+    }
+    #[test]
+    fn test_parse_1900s() {
         assert_eq!(
             Ssn::parse("010114A173M").unwrap(),
             Ssn {
@@ -500,6 +703,9 @@ mod tests {
                 gender: Gender::Male,
             }
         );
+    }
+    #[test]
+    fn test_parse_leap_year() {
         // pass when given valid finnishSSN with leap year, divisible only by 4
         assert_eq!(
             Ssn::parse("290296-7808").unwrap(),
@@ -510,10 +716,16 @@ mod tests {
                 gender: Gender::Female,
             }
         );
+    }
+    #[test]
+    fn test_parse_invalid_day_on_leap_year() {
         assert!(
             Ssn::parse("290200-101P").unwrap_err() == ParseError::Day("Invalid day number", 0, 2),
             "fail when given valid finnishSSN with leap year, divisible by 100 and not by 400"
         );
+    }
+    #[test]
+    fn test_parse_leap_year_long() {
         // pass when given valid finnishSSN with leap year, divisible by 100 and by 400
         assert_eq!(
             Ssn::parse("290200A248A").unwrap(),
@@ -524,10 +736,16 @@ mod tests {
                 gender: Gender::Female,
             }
         );
+    }
+    #[test]
+    fn test_parse_leading_whitespace() {
         assert!(
             Ssn::parse("010114A173M ").unwrap_err() == ParseError::Syntax("Invalid length", 0, 12),
             "fail when given SSN longer than 11 chars, bogus in the end"
         );
+    }
+    #[test]
+    fn test_parse_trailing_whitespace() {
         assert!(
             Ssn::parse(" 010114A173M").unwrap_err() == ParseError::Syntax("Invalid length", 0, 12),
             "fail when given SSN longer than 11 chars, bogus in the beginning"
@@ -542,16 +760,11 @@ mod tests {
 
     #[test]
     fn test_pattern_parse() {
-        assert!(
-            SsnPattern::parse("").unwrap_err() == ParseError::Syntax("Invalid length", 0, 0),
-            "fail when given empty String"
-        );
-        assert!(
-            SsnPattern::parse("123456X7890").unwrap_err()
-                == ParseError::Syntax("Invalid separator character", 6, 7),
-            "fail when separator is not valid character"
-        );
         assert!(SsnPattern::parse("123456-7890").is_ok(), "parse valid SSN");
+    }
+
+    #[test]
+    fn test_pattern_parse_all_wildcard() {
         assert!(
             SsnPattern::parse("??????-????").is_ok(),
             "parse all wildcard input"
@@ -559,11 +772,75 @@ mod tests {
     }
 
     #[test]
-    fn test_generate_pattern() {
+    fn test_pattern_parse_invalid_checksum() {
         assert!(
-            Ssn::generate_by_pattern(&SsnPattern::parse("111111-111?").unwrap()).unwrap()
-                == "111111-111C",
-            "generate expected SSN"
+            SsnPattern::parse("??????-???O").is_err(),
+            "invalid checksum"
         );
     }
+
+    use regex::Regex;
+
+    macro_rules! ssn_generate_success {
+        ($($name:ident: $value:expr,)*) => {$(
+            #[test]
+            fn $name() {
+                let pattern = &SsnPattern::parse($value).unwrap();
+                let generated = Ssn::generate_by_pattern(pattern).unwrap();
+                let matcher = Regex::new($value.replace("?", ".").as_str()).unwrap();
+                assert!(matcher.is_match(&generated), "retain expected values");
+                assert!(Ssn::parse(&generated).is_ok(), "generate valid SSN");
+            }
+        )*}
+    }
+
+    ssn_generate_success! {
+        first_day_of_year_wildcard: "010100-????",
+        first_day_of_year_fixed: "010100-???A",
+        identifier_smallest_wildcard: "???????002?",
+        identifier_smallest_fixed: "???????002A",
+        identifier_biggest_wildcard: "???????899?",
+        identifier_biggest_fixed: "???????899A",
+        decade_smallest_wildcard: "??????+????",
+        decade_smallest_fixed: "??????+???A",
+        decade_biggest_wildcard: "??????A????",
+        decade_biggest_fixed: "??????A???A",
+        month_smallest_wildcard: "??01???????",
+        month_smallest_fixed: "??01??????A",
+        month_biggest_wildcard: "??12???????",
+        month_biggest_fixed: "??12??????A",
+        day_smallest_wildcard: "01?????????",
+        day_smallest_fixed: "01????????A",
+        day_biggest_wildcard: "01?????????",
+        day_biggest_fixed: "01????????A",
+    }
+
+    macro_rules! ssn_generate_failure {
+        ($($name:ident: $value:expr,)*) => {$(
+            #[test]
+            fn $name() {
+                let pattern = &SsnPattern::parse($value).unwrap();
+                assert!(Ssn::generate_by_pattern(pattern).is_err());
+            }
+        )*}
+    }
+
+    ssn_generate_failure! {
+        identifier_too_small_wildcard: "???????001?",
+        identifier_too_small_fixed: "???????001A",
+        identifier_too_large_wildcard: "???????900?",
+        identifier_too_large_fixed: "???????900A",
+        month_too_small_wildcard: "??00???????",
+        month_too_small_fixed: "??00??????A",
+        month_too_large_wildcard: "??13???????",
+        month_too_large_fixed: "??13??????A",
+        day_too_small_wildcard: "00?????????",
+        day_too_small_fixed: "00????????A",
+        day_too_large_wildcard: "32?????????",
+        day_too_large_fixed: "32????????A",
+        day_too_large_on_non_leap_year_wilcard: "290299-????",
+        day_too_large_on_non_leap_year_fixed: "290299-???A",
+        day_too_large_on_leap_year_wildcard: "300204-????",
+        day_too_large_on_leap_year_fixed: "300204-???A",
+    }
 }
